capture log close
log using rm3ch6-binary2_v1, replace text

//  scott long & jeremy freese (jslong@indiana.edu & jfreese@northwestern.edu)
//  regression models for categorical outcomes using stata, 3rd
//  chapter 6: Models for binary outcomes: Interpretation | 2014-07-29

version 13.1
clear all
macro drop _all
program drop _all
set linesize 80
set scheme s2manual

//  6.1 Interpretation using regression coefficients

use binlfp4, clear
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog

pwcompare agecat, effects eform
listcoef, help
listcoef, reverse
listcoef, help percent

* (Advanced) Interpretation using y*
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
listcoef, std help

probit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
listcoef, std

//  6.2 Marginal effects: changes in probabilities

* Examples of marginal effects
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store base
estat summarize, labels

* Average marginal effects for continuous variables
mchange inc lwg
mchange inc, delta(5) brief
mchange k5, amount(one) brief
mchange k5 k618, amount(range) brief
mchange inc lwg, amount(range) trim(5) brief
centile inc lwg, centile(0 5 95 100)
mchange inc lwg, amount(range) trim(5) ///
    statistics(change from to pvalue) brief
mchange hc wc, stat(change from to pvalue) brief
mchange agecat, stat(change from to pvalue) brief
mchange
mchange, amount(one sd) decimals(2) brief

* Marginal effects for subgroups
mchange k5 if wc == 1, amount(one)

* Marginal effects at means and marginal effects at representative values
mchange, statistics(ci) atmeans
mchange k5, at(wc=1 hc=1) amount(1) atmeans
mchange inc, delta(5) amount(delta) ///
  at(inc=0 k5=2 wc=0 hc=0) atmeans

* Marginal effects with powers and interactions
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
mchange inc, atmeans

logit lfp c.inc c.inc#c.inc lwg k5 k618 i.agecat i.wc i.hc , nolog
mchange inc lwg, atmeans amount(sd)

* The distribution of marginal effects
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog

predict double prhat if e(sample)
gen double mcinc = prhat * (1-prhat) * _b[inc]
label var mcinc "Marginal change of inc on Pr(LFP)"
probit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
predict double pbtxb, xb
label var pbtxb "xb"
gen double pbtpdf = normalden(pbtxb)
label var pbtpdf "normal pdf at xb"
gen double pbtmcinc = pbtpdf * _b[inc]
label var pbtmcinc "Marginal change of inc from probit"

pwcorr pbtmcinc mcinc

quietly mchange inc
local ame = el(r(table),3,1)
quietly mchange inc, atmeans
local mem = el(r(table),3,1)

histogram mcinc, xlab(-.008(.002)0) ylab(0(.1).4,grid) ///
fraction bin(25) col(gs10) fcol(gs12) ///
text(.015 `ame' "AME", place(center)) ///
text(.000 `ame' "|", place(center)) ///
text(.015 `mem' "MEM", place(center)) ///
text(.000 `mem' "|", place(center))
graph export rm3ch6-marginal-dist-inc.emf,  replace

* discrete change computed using predict
gen wc_orig = wc // step 1
replace wc = 0 // step 2
predict double prhat_wc0 // step 3
replace wc = 1 // step 4
predict double prhat_wc1 // step 5
replace wc = wc_orig // step 6
gen double dc_wc = prhat_wc1 - prhat_wc0 // step 7
label var dc_wc "Discrete change of wc on Pr(LFP)"

* dc wc get AME and MEM
quietly mchange wc
local ame = el(r(table),1,1)
quietly mchange wc, atmeans
local mem = el(r(table),1,1)

* dc wc histogram
histogram dc_wc, xlab(0(.05).20) ylab(0(.1).3,grid) ///
    fraction bin(25) col(gs10) fcol(gs12) ///
    text(.015 `ame' "AME", place(center)) ///
    text(.000 `ame' "|", place(center)) ///
    text(.015 `mem' "MEM", place(center)) ///
    text(.000 `mem' "|", place(center))
graph export rm3ch6-discrete-dist-wc.emf,  replace

* (Advanced) Algorithm for computing the distribution of effects
use binlfp4, clear
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store mymodel
margins, dydx(wc)
matlist r(b)
margins, at(k5=0) at(k5=3) post
mlincom 2 - 1
estimates restore mymodel
estimates restore mymodel
margins, at() at(inc=gen(inc+1)) post
mlincom 2 - 1

* Algorithm for computing the distribution of marginal effects

// step 1: estimate the model
use binlfp4, clear
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
// step 2: create a variable containing e(sample)
tempvar insample
gen `insample' = e(sample)
label var `insample' "In estimation sample?"
// step 3: create a variable to hold effects
tempvar effect
gen `effect' = .
label var `effect' "Marginal effect for each observation"
// loop through all observations
local nobs = _N
forvalues i = 1/`nobs' {    
    if `insample'[`i']==1 { // step 4: only cases in estimation sample
        
        // step 5: use margins to compute effect for current case
        qui margins in `i', dydx(wc) nose
        
        // step 6: save marginal effect in variable
        qui replace `effect' = el(r(b),1,2) in `i'
    }
}
// step 7: compare average of effect variable to AME from margins
sum `effect'
margins, dydx(wc)
// step 8: plot the distribution of effects
histogram `effect', title(Distribution of marginal effects for wc)
graph export rm3ch6-me-distribution.emf, replace

//  6.3 Ideal types

use binlfp4, clear
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
mtable, rowname(1 Young low SES young kids) ci clear ///
    at(agecat=1 k5=2 k618=0 inc=10 lwg=.75 hc=0 wc=0)
mtable, rowname(2 Young college young kids) ci below ///
    at(agecat==1 k5==2 k618==0 wc==1 hc==1) atmeans
quietly mtable, rowname(3 Midage college with teens) ci below ///
    at(agecat==2 k5==0 k618==2 wc==1 hc==1) atmeans
mtable, rowname(4 Older college with adult kids) ci below ///
    at(agecat==3 k5==0 k618==0 wc==1 hc==1) atmeans

* Using local means with ideal types
capture drop _sel*
gen _selYC = agecat==1 & k5==2 & k618==0 & wc==1 & hc==1
label var _selYC "Select Young college young kids"
gen _selMC = agecat==2 & k5==0 & k618==2 & wc==1 & hc==1
label var _selMC "Select Midage college with teens"
gen _selOC = agecat==3 & k5==0 & k618==0 & wc==1 & hc==1
label var _selOC "Select Older college with adult kids"

quietly mtable, rowname(1 Young low SES young kids) ci clear ///
    at(agecat=1 k5=2 k618=0 inc=10 lwg=.75 hc=0 wc=0)
quietly mtable if _selYC==1, rowname(2 Young college young kids) ///
    atmeans ci below
quietly mtable if _selMC==1, rowname(3 Midage college with teens) ///
    atmeans ci below
mtable if _selOC==1, rowname(4 Older college with adult kids) ///
    atmeans ci below

* Comparing ideal types with statistical tests
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store base
mtable, atmeans post ///
    at(agecat=1 k5=2 k618=0 wc=0 hc=0 lwg=.75 inc=10) /// ideal type 1
    at(agecat=1 k5=2 k618=0 wc=1 hc=1 lwg=1.62 inc=16.64) // ideal type 2
mlincom 1 - 2
estimates restore base

* (Advanced) Using macros to test differences between ideal types
mtable, atmeans at(agecat=1 k5=2 k618=0 wc=0 hc=0 lwg=.75 inc=10)
display "`r(atspec)'"
local myatspec `r(atspec)'
mtable, atmeans at(`myatspec')

quietly mtable, atmeans at(agecat=1 k5=2 k618=0 inc=10 lwg=.75 hc=0 wc=0)
local YngLow `r(atspec)'
quietly mtable if _selYC == 1, atmeans
local YngCol `r(atspec)'
quietly mtable if _selMC == 1, atmeans
local MidCol `r(atspec)'
quietly mtable if _selOC == 1, atmeans
local OldCol `r(atspec)'

mtable, at(`YngLow') at(`YngCol') at(`MidCol') at(`OldCol') post
mlincom 1 - 2
mlincom 1 - 3
mlincom 1 - 4

estimates restore base
margins, at(`YngLow') at(`YngCol') at(`MidCol') at(`OldCol') ///
noatlegend pwcompare(effects)

* Marginal effects for ideal types
mchange wc k5, atmeans amount(one) at(`YngLow') 
matrix YngLow = r(table)
mchange wc k5, atmeans amount(one) at(`YngCol') 
matrix YngCol = r(table)
mchange wc k5, atmeans amount(one) at(`MidCol') 
matrix MidCol = r(table)
mchange wc k5, atmeans amount(one) at(`OldCol') 
matrix OldCol = r(table)

matrix me = YngLow[1...,1], YngCol[1...,1], MidCol[1...,1], OldCol[1...,1]
matrix colnames me = YngLow YngCol MidCol OldCol
matlist me, format(%9.2f) twidth(15)

// 6.4 Tables of predicted probabilities
use binlfp4, clear
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store base

mtable, at(wc=(0 1) k5=(0 1 2 3)) atmeans
mtable, at(wc=0 k5=(0 1 2 3)) at(wc=1 k5=(0 1 2 3)) atmeans ///
    atvars(wc k5) names(columns)
quietly mtable, estname(NoCol) at(wc=0 k5=(0 1 2 3)) atmeans brief
quietly mtable, estname(College) at(wc=1 k5=(0 1 2 3)) atmeans ///
    atvars(_none) right
mtable, estname(Change) dydx(wc) at(k5=(0 1 2 3)) atmeans ///
    atvars(_none) right stats(estimate p) names(columns) brief
mtable if agecat == 1, estname(Change) dydx(wc) at(k5=(0 1 2 3)) atmeans ///
    atvars(k5) stats(estimate p) names(columns)

quietly mtable if agecat==1, estname(NoCol) at(wc=0 k5=(0 1 2 3)) atmeans
quietly mtable if agecat==1, estname(College) at(wc=1 k5=(0 1 2 3)) atmeans ///
    atvars(_none) right
mtable if agecat==1, estname(Change) dydx(wc) at(k5=(0 1 2 3)) atmeans ///
    atvars(_none) right stats(estimate p) names(columns) brief

mtable, at(wc=(0 1)) atmeans over(k5) brief

qui mtable, estname(Wife:NoCol)   at(wc=0) over(k5) atmeans atvars(k5)
qui mtable, estname(Wife:College) at(wc=1) over(k5) atmeans atvars(_none) right
mtable, estname(Change) dydx(wc) over(k5) atmeans ///
    atvars(_none) right stats(est p) names(columns)
matrix k5wc_localk5 = _mtab_display

* global and local comparison table
/*
matrix mk5 = k5wc_global[1...,1]
matrix mg = k5wc_global
matrix ml = k5wc_localk5
matrix coleq mg = Global
matrix coleq ml = Local
matcol mg keep 2 3 4
matcol ml keep 2 3 4
matmath mdif = mg - ml
matrix coleq mdif = "G - L"
matrix colnames mdif = NoCol    College       Chng
matrix mcompare = mg, ml, mdif
matrix colnames mcompare = NoCol Col Chng NoCol Col Chng NoCol Col Chng
matrix mcompare = mk5, mcompare
local sjis mtable_local_compareEDIT
matlist mcompare, names(col) format(%5.2f)
*/

//  6.5 Second differences comparing marginal effects
logit lfp k5 k618 i.agecat wc##hc lwg inc, nolog
estimates store base

mchange wc if hc==1
mchange wc if hc==0

mtable, dydx(wc) over(hc) post
test _b[1.wc:0.hc]=_b[1.wc:1.hc]
mlincom 1 - 2

estimates restore base
mtable, dydx(wc) over(hc) stat(ci) post
mlincom 1 - 2

//  6.6 Graphing predicted probabilities

use binlfp4, clear
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store base

* Using marginsplot
margins, at(inc=(0(10)100)) atmeans noatlegend
mlistat
marginsplot
graph export rm3ch6-marginsplot-inc.emf, replace

* Using mgen with the graph command
capture drop PLT*
mgen, atmeans at(inc=(0(10)100)) stub(PLT) predlabel(Pr(LFP))
list PLTinc PLTpr PLTll PLTul lfp in 1/13, clean

scatter PLTpr PLTinc
graph export rm3ch6-mgen-inc-simple.emf, replace

twoway ///
    (rarea PLTul PLTll PLTinc, color(gs12)) ///
    (connected PLTpr PLTinc, msymbol(i)) ///
    , title("Adjusted Predictions") ///
    caption("Other variables held at their means") ///
    ytitle(Pr(LFP)) ylabel(0(.25)1, grid gmin gmax) legend(off)
graph export rm3ch6-mgen-inc-ciarea.emf, replace

* Graphing multiple predictions

margins agecat, at(inc=(0(10)100)) atmeans noatlegend
mlistat

marginsplot, noci legend(cols(3))
graph export rm3ch6-marginsplot-inc-noci.emf, replace


mgen, atmeans at(inc=(0(10)100) agecat=1) stub(PLT1) predlab(30 to 39)
mgen, atmeans at(inc=(0(10)100) agecat=2) stub(PLT2) predlab(40 to 49)
mgen, atmeans at(inc=(0(10)100) agecat=3) stub(PLT3) predlab(50 plus)

graph twoway connected PLT1pr PLT2pr PLT3pr PLT1inc, ///
    title("Adjusted predictions by age group") ///
    caption("Other variables at their means") ///
    msym(Oh Dh Sh) msiz(*1.4 *1.1 *1.2) mcol(black black black) ///
    lpat(solid solid solid) ///
    ytitle("Pr(In Labor Force)") ylab(0(.25)1, grid gmin gmax) ///
    legend(cols(3))
graph export rm3ch6-mgen-incXage.emf, replace

* Overlapping confidence intervals

mgen, atmeans at(inc=(0(5)100) wc=0) stub(PLTWC0) predlab(NoCollege)
mgen, atmeans at(inc=(0(5)100) wc=1) stub(PLTWC1) predlab(College)

twoway ///
    (rarea PLTWC0ul PLTWC0ll PLTWC0inc, col(gs12)) ///
    (rarea PLTWC1ul PLTWC1ll PLTWC0inc, col(gs12)) ///
    (connected PLTWC0pr PLTWC1pr PLTWC1inc, msym(i i)) ///
    , ytitle(Pr(In Labor Force)) legend(order(4 3))
graph export rm3ch6-mgen-incXwc-prob.emf, replace

mgen, dydx(wc) atmeans at(inc=(0(5)100)) stub(PLTWCDC) ///
    predlab(Discrete change in LFP by attending college)
twoway ///
    (rarea PLTWC0ul PLTWC0ll PLTWC0inc, col(gs12)) ///
    (rarea PLTWC1ul PLTWC1ll PLTWC0inc, col(gs12)) ///
    (connected PLTWC0pr PLTWC1pr PLTWC1inc, msym(i i)) ///
    , ytitle(Pr(LFP)) /// 
    xlin(2 39) ///
    legend(order(4 3)) saving(temppr, replace)
graph export rm3ch6-mgen-incXwc-X.emf, replace

* Adding power terms and plotting predictions
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
mgen, predlabel(linear) atmeans at(inc=(0(10)100)) stub(_lin)

logit lfp k5 k618 i.agecat i.wc i.hc lwg c.inc##c.inc, nolog
mgen, predlabel(quadratic) atmeans at(inc=(0(10)100)) stub(_quad)

graph twoway connected _linpr _quadpr _lininc, ///
    title("Comparing income specifications") ///
    caption("Other variables at their means") ///
    msym(Oh Dh) msiz(*1.5 *1) mcol(black black) lpat(solid dash) ///
    ytitle("Pr(In Labor Force)") ///
    ylabel(0(.25)1, grid gmin gmax)
graph export rm3ch6-mgen-incsq.emf, replace

* (Advanced) Graphs with local means
mgen, at(inc=(0(10)100)) atmeans stub(GLOBAL) predlabel(Global means)
gen inc10k = trunc(inc/10)
label var inc10k "income in 10K categories"
tabulate inc10k, miss

* mtable at each level of income, save as below 

* mtable if inc10k==0, atmeans ci
* mtable if inc10k==1, atmeans ci
* mtable if inc10k==2, atmeans ci
* mtable if inc10k==3, atmeans ci
* mtable if inc10k==4, atmeans ci
* mtable if inc10k==5, atmeans ci
* mtable if inc10k==6, atmeans ci
* mtable if inc10k==7, atmeans ci
* mtable if inc10k==8, atmeans ci
* mtable if inc10k==9, atmeans ci

mtable, over(inc10k) atmeans ci
matrix localpred = r(table)
matlist localpred, format(%8.2g)
matrix localpred = r(table)
matrix localpred = localpred[1...,"inc".."ul"]
matrix colnames localpred = LOCALinc LOCALpr LOCALll LOCALul
svmat localpred, names(col)
label var LOCALpr "Local means"

twoway ///
    (connected GLOBALpr GLOBALinc, ///
    clcol(black) clpat(solid) msym(i)) ///
    (connected LOCALpr LOCALinc, ///
    clcol(black) clpat(dash) msym(i)) ///
    , ytitle("Pr(In Labor Force)") ylab(0(.25)1, grid gmin gmax)
graph export rm3ch6-mgen-inc-glocal-local.emf, replace

log close
exit
